$typecheck on
'============================================================================================
'  Class QFILEDIAGLOG   an Open or Save sizeable file dialog box with overwrite warning
'  Version 1.0 
'  11/2004 original code adapted from RQDIALOG.INC by   BILL K  1-2003
'  by JohnK
'============================================================================================
' USAGE:  MYFILENAME$=RQSHOWDIALOG(MyCaption$, _                ' default is "Open" or "Save"
'                                  MyFilter$, _                 ' default is All Files
'                                  FilterIndex%, _              ' default is 0
'                                  MyInitialDirectory$, _       ' default is current directory
'                                  WantSaveDialog%, _           ' 1=Save  0=Open - default is Open 
'                                  NoOverwriteWarning%, _       ' default is 0
'                                  MyFormHandle&)               ' Form handle or 0 .. default is 0 - see below
' 
'    WantSaveDialog%=1 for a Save Dialog   =0 for an Open Dialog
'    NoOverwriteWarning%=1 for defeating the default warning when about to overwrite an existing file
'    MyFormHandle& = MyForm.Handle ( so that the dialog knows which form
'                                    to be modal to and where to locate)   =0 for nonmodal(not advised)
' 
'  EXAMPLE: FileName$=RQShowDialog("Save File","RQ Files|*.bas;*.rqb","C:\RapidQ",1,0,Form.Handle)
'
$IFNDEF _RQDIALOGINC
$DEFINE _RQDIALOGINC
     
CONST OFN_READONLY                 =  &H00000001  
CONST OFN_OVERWRITEPROMPT          =  &H00000002  'display the overwrite warning
CONST OFN_HIDEREADONLY             =  &H00000004  'hide the Open as Readonly checkbox
CONST OFN_NOCHANGEDIR              =  &H00000008  'return to original director if the user changed the directory while searching
CONST OFN_SHOWHELP                 =  &H00000010
CONST OFN_ENABLEHOOK               =  &H00000020
CONST OFN_ENABLETEMPLATE           =  &H00000040
CONST OFN_ENABLETEMPLATEHANDLE     =  &H00000080
CONST OFN_NOVALIDATE               =  &H00000100
CONST OFN_ALLOWMULTISELECT         =  &H00000200   'multiple file select, should have OFN_EXPLORER = 1 for long filenames
CONST OFN_EXTENSIONDIFFERENT       =  &H00000400
CONST OFN_PATHMUSTEXIST            =  &H00000800
CONST OFN_FILEMUSTEXIST            =  &H00001000
CONST OFN_CREATEPROMPT             =  &H00002000
CONST OFN_SHAREAWARE               =  &H00004000
CONST OFN_NOREADONLYRETURN         =  &H00008000
CONST OFN_NOTESTFILECREATE         =  &H00010000
CONST OFN_NOLONGNAMES              =  &H00040000
CONST OFN_EXPLORER                 =  &H00080000	'explorer like browser
CONST OFN_NODEREFERENCELINKS       =  &H00100000
CONST OFN_LONGNAMES                =  &H00200000



'Error codes from CommDlgExtendedError
Public Const CDERR_DIALOGFAILURE = &HFFFF
Public Const CDERR_GENERALCODES = &H0
Public Const CDERR_STRUCTSIZE = &H1
Public Const CDERR_INITIALIZATION = &H2
Public Const CDERR_NOTEMPLATE = &H3
Public Const CDERR_NOHINSTANCE = &H4
Public Const CDERR_LOADSTRFAILURE = &H5
Public Const CDERR_FINDRESFAILURE = &H6
Public Const CDERR_LOADRESFAILURE = &H7
Public Const CDERR_LOCKRESFAILURE = &H8
Public Const CDERR_MEMALLOCFAILURE = &H9
Public Const CDERR_MEMLOCKFAILURE = &HA
Public Const CDERR_NOHOOK = &HB
Public Const CDERR_REGISTERMSGFAIL = &HC



TYPE OPENFILENAME
	lStructSize 	AS DWORD	'Specifies the length, in bytes, of the structure. 
	hwndOwner 		AS LONG		'any valid window handle, or it can be NULL if the dialog box has no owner
	hInstance 		AS LONG		'for OFN_ENABLETEMPLATEHANDLE or ENABLETEMPLATE  flags
	lpstrFilter 	AS LONG		'pointer to string containing pairs of filter strings, ("*.TXT;*.DOC;*.BAK"), no spaces!
	lpstrCustomFilter AS LONG	'null or pointer to a pair of null-terminated filter strings for preserving the filter pattern chosen by the user
	nMaxCustFilter 	AS DWORD	'the size, in bytes of the buffer identified by lpstrCustomFilter
	nFilterIndex 	AS DWORD	'index of the currently selected filter in the File Types 
	lpstrFile 		AS LONG		'Pointer to string of the file name to initialize control,can be multple if OFN_ALLOWMULTISELECT  set, _
								'first char must be NULL if no initialization. Returns drive designator, path, file name, and extension of the selected file. 
	nMaxFile 		AS DWORD	'size in bytes of lpstrFile should be >256 chars
	lpstrFileTitle 	AS LONG		'Pointer to string that receives the file name and extension (without path information) of the selected file. This member can be NULL
	nMaxFileTitle 	AS DWORD	'size in bytes of lpstrFileTitle
	lpstrInitialDir AS LONG		'pointer to string containing directory, different effect on different OS versions!
	lpstrTitle 		AS LONG		'pointer to string containing title
	flags 			AS DWORD	'
	nFileOffset 	AS SHORT	'zero-based offset, number of chars  to the 1st character of the file name in lpstrFile (not path)
	nFileExtension 	AS SHORT	'zero-based offset, number of chars  to the 1st character of the file extension in lpstrFile (not path)
	lpstrDefExt 	AS LONG		'pointer to string to append as filename extension if the user fails to type an extension- only 3 characters are used, should not contain a period (.)
	lCustData 		AS LONG		'AS LPARM
	lpfnHook 		AS LONG		'AS LPOFNHOOKPROC, pointer to callback function
	lpTemplateName 	AS LONG		'pointer to string that names a dialog template resource in the module identified by the hInstance member
'#if (_WIN32_WINNT >= 0x0500)
'  void *        pvReserved;
'  DWORD         dwReserved;
'  DWORD         FlagsEx;
'#endif // (_WIN32_WINNT >= 0x0500)
END TYPE

DECLARE FUNCTION GetSaveFileName LIB "COMDLG32" ALIAS "GetSaveFileNameA" (pOpenfilename AS OPENFILENAME) AS LONG
DECLARE FUNCTION GetOpenFileName LIB "COMDLG32" ALIAS "GetOpenFileNameA" (pOpenfilename AS OPENFILENAME) AS LONG
Declare Function CommDlgExtendedError Lib "comdlg32.dll" Alias "CommDlgExtendedError" () As Long


CONST qfdOpen = 0
CONST qfdSave = 1
CONST pdfStrTerm$ = CHR$(0) + CHR$(0)
CONST qfdMaxFiles = 1000

TYPE QFILEDIALOG EXTENDS QOBJECT
  PRIVATE:
	tmpFileName		AS STRING
	tmpFilter		AS STRING
	tmpFileTitle	AS STRING		'output string
	OFN				AS OPENFILENAME

  PUBLIC:
	Parent			AS LONG			'Parent form handle, can be NULL
	Caption 		AS STRING		' default is "Open" or "Save"
	Filter			AS STRING		'*.*
	FilterIndex		AS INTEGER		'0 based
	InitialDir		AS STRING
	Mode			AS INTEGER		'enum qfdOpen = Open (default),  qfdSave = save,
	WarnIfOverWrite	AS INTEGER		'warning to overwrite?
	Filename		AS STRING
	FileTitle		AS STRING
	DefaultExt		AS STRING
	MultiSelect 	AS INTEGER
	SelCount 		AS INTEGER
	NoChangeDir		AS INTEGER
	Files(qfdMaxFiles) AS STRING

 CONSTRUCTOR
	Parent			= 0			'Parent form handle, can be NULL
	Caption 		= "Open"
	Filter			= "All Files|*.*" + pdfStrTerm$
	FilterIndex		= 0
	InitialDir		= ""
	Mode			= 0		'enum 0 = Open (default), 1 = save,
	WarnIfOverWrite	= 1		'warning to overwrite?
	Filename		= ""
	MultiSelect 	= 0
	SelCount 		= 0
	NoChangeDir		= 0
 END CONSTRUCTOR


 FUNCTION Execute() AS LONG
  DIM FndSpace1 	AS INTEGER
  DIM FndSpace2 	AS INTEGER
  DIM StrLenth		AS INTEGER
  DIM MyHresult		AS LONG			'return result

  WITH QFILEDIALOG
	.OFN.lStructSize = SIZEOF(QfileDialog.OFN)
	.OFN.hwndOwner = .Parent
	.OFN.hInstance = 0

	'file filter string
	IF RIGHT$(.Filter, 2) <> pdfStrTerm$ THEN .Filter = .Filter + pdfStrTerm$
	.tmpFilter = REPLACESUBSTR$(.Filter, "|", CHR$(0)) 
'	.tmpFilter = REPLACESUBSTR$(.Filter, " ","")		'no spaces
	.OFN.lpstrFilter = VARPTR(QfileDialog.tmpFilter)
	.OFN.nFilterIndex = .FilterIndex

	IF .FileName = "" THEN 
		.tmpFileName = String$(1024,0)
		.OFN.nMaxFile = 1024
	ELSE
		.tmpFileName = .FileName
		IF LEN(.FileName) < 1024 THEN .tmpFileName = .tmpFileName + String$(1024-LEN(.FileName),0)	' must be > 256
		.OFN.nMaxFile = LEN(.tmpFileName)
	END IF

	.OFN.lpstrFile = VARPTR(QfileDialog.tmpFileName)
	.tmpFileTitle = String$(1024,0)				'string that gets the result,without path
	.OFN.nMaxFileTitle = 1024'LEN(.tmpFileTitle)
	.OFN.lpstrFileTitle = VARPTR(QfileDialog.tmpFileTitle)
	.OFN.lpstrInitialDir = VARPTR(QfileDialog.InitialDir)
	.OFN.lpstrTitle = VARPTR(QfileDialog.Caption)
	IF .DefaultExt <> "" THEN .OFN.lpstrDefExt = VARPTR(QfileDialog.DefaultExt)

	IF .WarnIfOverWrite THEN
		.OFN.flags =OFN_HIDEREADONLY OR OFN_OVERWRITEPROMPT
	ELSE
		.OFN.flags =OFN_HIDEREADONLY
	END IF
	IF .MultiSelect THEN .OFN.flags = .OFN.flags OR OFN_ALLOWMULTISELECT OR OFN_EXPLORER
	IF .NoChangeDir THEN .OFN.flags = .OFN.flags OR OFN_NOCHANGEDIR


	IF .Mode = qfdOpen THEN
		MyHresult = GetOpenFileName(QfileDialog.OFN)
	ELSE
		IF .Mode = qfdSave THEN
			MyHresult = GetSaveFileName(QfileDialog.OFN)
		ELSE
			ShowMessage "Invalid Mode in FileDialog"
		END IF
	END IF

	IF MyHresult <> 0 THEN								'returns 0 on CANCEL
		.tmpFileName = STRING$(.OFN.nMaxFile, 32)		'allocate string memory
		.FileName = STRING$(.OFN.nMaxFile, 32)			'allocate string memory in case
		MEMCPY(VARPTR(QfileDialog.tmpFileName), QfileDialog.OFN.lpstrFile, QfileDialog.OFN.nMaxFile)	'get string
		StrLenth = (INSTR(QfileDialog.tmpFileName, pdfStrTerm$))
		.tmpFileName = LEFT$(QfileDialog.tmpFileName, StrLenth)
		.FileName = .tmpFileName
		IF .MultiSelect THEN
			.tmpFileName = REPLACESUBSTR$(.tmpFileName, CHR$(0), CHR$(2)) 			'prevent string termination
			FndSpace1 = 1: 	FndSpace2 = 0
			.SelCount = -1								'for 0 indexing
			DO
				FndSpace2 = INSTR(FndSpace1, .tmpFileName, CHR$(2))		'null doesn't work, trick w/chr 2
				IF FndSpace2 = 0 THEN
					 EXIT DO
				ELSE
					.SelCount++
					.Files(.SelCount) = MID$(.tmpFileName, FndSpace1, (FndSpace2 - FndSpace1))
					FndSpace1 = FndSpace2 + 1
					IF .SelCount > qfdMaxFiles THEN EXIT DO		'too many files
				END IF
			LOOP UNTIL FndSpace2 >= StrLenth
		ELSE
			.FileTitle = STRING$(.OFN.nMaxFileTitle, 32)		'allocate
			MEMCPY(VARPTR(QfileDialog.FileTitle), QfileDialog.OFN.lpstrFileTitle, QfileDialog.OFN.nMaxFile)
			.FileTitle = LEFT$(.FileTitle, (INSTR(.FileTitle, pdfStrTerm$))-1)
		END IF
	 END IF
	RESULT = MyHresult
   END WITH
  END FUNCTION

END TYPE
$ENDIF ' _RQDIALOGINC



dim i as integer
DIM fDialog as qfiledialog
fDialog.Caption = "QfileDialog test"
fDialog.FileName= "test.txt"
fDialog.InitialDir = "C:\RapidQ"
fDialog.MultiSelect = 1
IF fDialog.Execute THEN
IF fDialog.MultiSelect THEN
	for i = 0 to fDialog.SelCount
	ShowMessage fDialog.Files(i)	'string #0 is the directory
	next i
ELSE
	ShowMessage "file name & path =" + fDialog.FileName
	ShowMessage "file name no path =" + fDialog.FileTitle
END IF
END IF


